# Feb 5, 2017#
# Use a function to fit a tree with training and testing data sets
# return the root node with all tree structures
CITree <- function(Ytrain, Xtrain, Atrain, PTtrain, Ytest, Xtest, Atest, PTtest, maxL, iter.Max=20, C, s, tol=.0001){
  fit <- RAMP_SVM(Xtrain, Atrain, Ytrain, PTtrain, C, s, newX=X, iter.Max, tol)
  ITRtrain <- RAMP_SVM.pred(fit, Xtrain)
  ITRtest <- RAMP_SVM.pred(fit, Xtest)
  benefitA <- ITR_ben(Ytest, PTtest, Atest, ITRtest)
  print(benefitA)
  root <- InternalA$new(trainInd=rep(1, length(Ytrain)), testInd=rep(1, length(Ytest)), ITRtrain=ITRtrain, ITRtest=ITRtest, beta=c(fit$e.beta0,fit$e.beta), benefit=benefitA, candidate = rep(1, ncol(Xtrain)), nodeName="1" ) 
  nodeA_todo <- list(root)
  l <- 1
  while (l < maxL){
    nodeB_todo <- nodeA_to_B(nodeA_todo, Ytrain, Xtrain, Atrain, PTtrain, Ytest, Xtest, Atest, PTtest) 
    l <- l + 1
    if(length(nodeB_todo)==0|l >= maxL){break}
    nodeA_todo <- nodeB_to_A(root, nodeB_todo, Ytrain, Xtrain, Atrain, PTtrain, Ytest, Xtest, Atest, PTtest)
    l <- l + 1
    if(length(nodeA_todo)==0|l >= maxL){break}
  }
  return(list(root=root, Level=l))
}

#Feb 21 update: summarize the CITree by nodes
CITree.summary <- function(node){
  out = NULL
  currentNodes = list(node)
  l = 1
  while (length(currentNodes) != 0) {
    tempNodes = list()
    for (i in 1:length(currentNodes)){
      termNode = 0
      #for odds level report beta
      if (l%%2 !=0) {
        splitvar = NA; splitpoint = NA;
        beta0=currentNodes[[i]]$beta[1];
        beta=currentNodes[[i]]$beta[-1]
        if (class(currentNodes[[i]]$left) == "Tree") {
          termNode =1;
        }
      } else if (class(currentNodes[[i]]$cutoff) == "uninitializedField"){
        splitvar = NA; splitpoint = NA;
        beta0 = NA; beta = rep(NA, length(currentNodes[[i]]$beta)-1);
        termNode = 1
      } else{ #report split var and point for even levels
        splitvar = currentNodes[[i]]$cutoff[1];
        splitpoint = currentNodes[[i]]$cutoff[2];
        beta0 = NA; beta = rep(NA, length(currentNodes[[i]]$beta)-1)
      }
      out = rbind(out, c(nodeName = as.numeric(currentNodes[[i]]$nodeName), 
                       train.size = sum(currentNodes[[i]]$trainInd),  
                       test.size = sum(currentNodes[[i]]$testInd), 
                       nodeBenefit = currentNodes[[i]]$benefit,
                       termNode = termNode,
                       splitvar = splitvar, 
                       splitpoint = splitpoint,
                       beta0 = beta0,
                       beta = beta)
                  )
      if(class(currentNodes[[i]]$left) != "Tree"){
        tempNodes = c(tempNodes, currentNodes[[i]]$left)
      }
      if(class(currentNodes[[i]]$right) != "Tree"){
        tempNodes = c(tempNodes, currentNodes[[i]]$right)
      }
    }
    currentNodes = tempNodes
    l = l + 1
  }
  return(as.data.frame(out))
}


# predict the ITR using CITree for a new set of subjects
# Return with predicted optimal treatment and average benefit of the terminal node
pred <- function(node, x){
  result_node = NULL;
  while (length(node) != 0) {
    if (sign(node$beta %*% c(1, x))<=0){
      pred.ITR <- -1;
      if(class(node$left) == "Tree"){
        result_node = node;
        node = NULL;
      } else if(class(node$left$left) == "Tree"){
        result_node = node$left;
        node = NULL;
      } else {
        if(x[node$left$cutoff[1]] <=node$left$cutoff[2]){
          node = node$left$left
        } else {
          node = node$left$right
        }
      }       
    } else{
      pred.ITR <- 1;
      if(class(node$right) == "Tree"){
        result_node = node;
        node = NULL
      } else if(class(node$right$left) == "Tree"){
        result_node = node$right;
        node = NULL;
      } else {
        if(x[node$right$cutoff[1]] <= node$right$cutoff[2]){
          node = node$right$left
        } else {
          node = node$right$right
        }
      }
    }
  }
 return (list(predITR = pred.ITR, benefit=result_node$benefit, termNode= as.numeric(result_node$nodeName)))
 #return(pred.ITR)
}
  
  
# predict the ITR using CITree for a new set of subjects
# Return with predicted optimal treatment and average benefit of the terminal node
CITree.predict <- function(node, newX){
  ITR <- rep(NA, dim(newX)[1])
  benefit <- rep(NA, dim(newX)[1])
  termNode <- rep(NA, dim(newX)[1])
  for (i in 1:dim(newX)[1]){
    ITR[i] <- pred(node, newX[i,])$predITR
    benefit[i] <- pred(node, newX[i,])$benefit
    termNode[i] <- pred(node, newX[i,])$termNode
  }
  return(list(ITR=ITR, benefit=benefit, termNode=termNode))
}

