### Updated on Nov 14: modify the function to calculate FDR and PPV of HTB test###
#In the bin_split and best_split function, return all results instead of best row#

#####################################################
##### Adjust for multiple cut points using FDR ######
#####################################################


#Calculate ITR benefit given assigned and predicted treatment for any subgroup 
ITR_ben <- function(Y, PrTx, A, ITR){
  mean(Y*(A==ITR)/PrTx)-mean(Y*(A!=ITR)/PrTx)
}


###Updated on Nov 13: using function delta method to estimate var and cov####

HTB <- function(x, splitpoint, A, ITR, PrTx, Y){  
  Index <- x <= splitpoint
  Indexc <- x > splitpoint
  n1 <- sum(Index)
  n2 <- sum(Indexc)
  n <- n1 + n2
  if (sum(n1)>0 & sum(n2)>0){
  
    fn <- (Index==1 & A== ITR)*Y/PrTx - (Index==1 & A!= ITR)*Y/PrTx
    fn_ <- (Indexc==1 & A== ITR)*Y/PrTx - (Indexc==1 & A!= ITR)*Y/PrTx
    
    gn <- Index==1
    gn_ <- Indexc==1
    
    
    Efn <- mean(fn)
    Egn <- mean(gn)
    Efn_ <- mean(fn_)
    Egn_ <- mean(gn_)
    
    var1 <- var(fn)/Egn^2 + Efn^2*var(gn)/Egn^4-2*Efn*cov(fn, gn)/Egn^3
    var2 <- var(fn_)/Egn_^2 + Efn_^2*var(gn_)/Egn_^4-2*Efn_*cov(fn_, gn_)/Egn_^3
    #cov12 <- (cov(a-b, c-d) + n*mu1*mu3)/(cov(e,f)+ n*mu2*mu4 )-mu1*mu3/mu2/mu4
    cov12 <- cov(fn, fn_)/Egn/Egn_ - cov(fn, gn_) * Efn_ /Egn/Egn_^2 - cov(fn_, gn) *Efn/Egn^2/Egn_ + cov(gn, gn_) * Efn * Efn_ / Egn^2/Egn_^2
    
    HTB_var = var1/n + var2/n - 2*cov12/n
    
    #ITR benefit for parent node and left node
    ben_all <- ITR_ben(Y, PrTx, A, ITR)
    ben_l <- ITR_ben(Y[Index], PrTx[Index], A[Index], ITR[Index])
    ben_r <- ITR_ben(Y[Indexc], PrTx[Indexc], A[Indexc], ITR[Indexc])
    
    HTB_stat <- (ben_l - ben_r)^2/HTB_var
    #follows chisquare df=1
    HTB_p <- 1-pchisq(HTB_stat,1)
    return(c(HTB_p, HTB_var, ben_l , ben_r , HTB_stat, n1 ,n2 ))
  }
}

#find the best split point for a variable
bin_split <- function(x, A, ITR, PrTx, Y){
  #use the 25%, 50% 75% quantile as candidate cutoff
  #z <- unique(quantile(x)[c(-1,-5)])
  #z <- seq(-0.5, 0.5, by = 0.1)
  #consider x with only 2 values
  if (length(unique(x)) >= 10){
    z <- seq(round(quantile(x)[2], 1), round(quantile(x)[4], 1), by=0.1)
  # z <- c(median(x), median(x))
  } else {
    z <- unique(x)
    z <- z[!z %in% max(z)] #remove the maximal for categorial case
  } 
    
  n <- length(z) 
  #n is  the number of distinct values (split points) of predictor x
  split_sum <- matrix(NA, nrow = n, ncol = 8)
  #Perform for each possible split point:
  for  (i in 1:n){
    splitpoint<-z[i]
    HTB <- HTB(x, splitpoint, A, ITR, PrTx, Y)
    #split_sum[i,] <- c(z[i], HTB$p, HTB$ben_l, HTB$ben_r, HTB$n1, HTB$n2)
    split_sum[i,] <- c(z[i], HTB)
  }
  #remove split points with 'NA' p-values 
  split_sum <- split_sum[!is.na(split_sum[,2]), ]
  #bestrow <- which(split_sum[,2]==min(split_sum[,2]))[1]
  #return(split_sum[bestrow,])
  #return all the rows for calculating PPV and FDR
  return (split_sum)
}

#best_split <- t(sapply(1:10,FUN = function(kk, X, A, ITR, PrTx, Y){
#  bin_split(x=X[,kk], A, fit$ITR, PrTx, Y)},
#  X=X, A=A, ITR=fit$ITR, PrTx=PrTx, Y=Y
#))
# find the best variable for splitting
# X is matrix of feature variables
# candidate is an index vector of X: indicating which variable will be included for searching best split

#FDR adjusted
best_split <- function(X, candidate, A, ITR, PrTx, Y){
  #initial value 
  #summary <- matrix (10, nrow = dim(X)[2], ncol = 9)
  summary <- NULL
  adjusted.p <- NULL
  for (j in 1:dim(X)[2]){
    # for candidte variable with more than 1 unique values
    if (candidate[j]==1 & length(unique(X[,j]))>1){
      #summary[j, ] <- c(j, bin_split(x=X[,j], A, ITR,  PrTx, Y))
      result <- bin_split(x=X[,j], A, ITR,  PrTx, Y)  
      #checking for median only: only one split point for a variable
      if (is.vector(result)){
        summary <- rbind(summary, cbind(j, t(result)))
      } else{ # more than one splitpoint for each variable
        summary <- rbind(summary, cbind(j, result))
      }
      #adjusted p within each variable
      adjusted.p <- c(adjusted.p, p.adjust(summary[which(summary[,1]==j),3], method="BH", n=length(summary[,3])))    
    }      
  }
  summary <- cbind(summary, adjusted.p)
  #print(summary)
  #return the row with smallest unadjusted p-value
  #return all the split results for estimating FDR/PPV
  bestrow <- which(summary[,3]==min(summary[,3]))
  #update March16, 2017, when adjusted p-values are equal 
  #pick the one with smallest HTB test stat not p-value
  #For eadge cases, all the split point have p-value=0

#   print(is.vector(bestrow))
#   print(is.matrix(bestrow))
  if(length(bestrow)>1){
     bestrow <- which(summary[,7] == max(summary[,7]))[1]
   }
  #return the smallest adjusted p-value
  return(list(best=summary[bestrow,], adjusted.p=min(summary[,ncol(summary)])))
}
